%%%%%%%%%%%%%%%%% Example 10.5 %%%%%%%%%%%%%%%%%%
%   Discrete-Time Control Problems using        %
%       MATLAB and the Control System Toolbox   %
%   by J.H. Chow, D.K. Frederick, & N.W. Chbat  %
%         Brooks/Cole Publishing Company        %
%                September 2002                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   ---- Observer design ----
%
clear
disp('Example 10.5')

Ts = 0.1;
A = [0.81   -0.23   -0.045
     0.09    0.98   -0.0023
     0.005   0.10    1     ];      % state matrix
B = [0.09 0.0047 0.00016]';        % input matrix
C = [1 3.5 3];                     % output matrix
D = 0;                             % throughput matrix
p = [0.50 0.60 0.75]';             % desired observer pole locations
L = place(A',C',p)'                % estimator gain
A_ob = A-L*C                       % observer system matrix
eig(A_ob)                          % check eigenvalues
disp('******>'), pause

x0 = [1; -0.75; 0.5]               % initial conditions
dtime = [0:Ts:6];                  % time array
u = zeros(1,length(dtime));        % zero input
G = ss(A,B,C,D,Ts)                 % build system as LTI object
[y,dtime,x] = lsim(G,u,dtime,x0);  % simulate system to get y(k)
disp('******>'), pause

disp('Simulate Observer')
G_ob = ss(A_ob,L,C,D,Ts)           % build system as LTI object
[y_hat,dtime,x_hat]=lsim(G_ob,y,dtime); % simulate observer with zero ICs
% plot system and observer states
figure
plot(dtime,x,'o',dtime,x_hat(:,1),'+',dtime,x_hat(:,2),'x',...
		dtime,x_hat(:,3),'^'),grid  % figure 10.1(a)
title('Observer design for Example 10.5')
xlabel('Time (s)')
ylabel('Amplitude')
text(0.6,0.75,'x_1')
text(0.30,-0.50,'x_2')
text(0.07,0.50,'x_3')
legend('x_1','x_2','x_3','x_1hat','x_2hat','x_3hat') % add legend
disp('******>'), pause

figure
plot(dtime,y,'o',dtime,y_hat,'*'), grid % figure 10.1(b)
title('Ex 10.5: Observer design')
xlabel('Time (s)')
ylabel('Amplitude')
legend('y','yhat')                  % add legend
%%%%%%%%%%