%%%%%%%%%%% Comprehensive Problem 8.2 %%%%%%%%%%%
%   Discrete-Time Control Problems using        %
%       MATLAB and the Control System Toolbox   %
%   by J.H. Chow, D.K. Frederick, & N.W. Chbat  %
%         Brooks/Cole Publishing Company        %
%                September 2002                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inverted Pendulum
% System has 4 states, 1 input, and 2 outputs
% 4 states : pendulum's angular position and velocity, and
%            cart's position and velocity
% 1 input  : motor voltage (u)
% 2 outputs: pendulum's angular position (theta)
%            cart's position (x) 

disp('CP8.2: Inverted Pendulum')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Part A : Load model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear; close all

% Load continuous and discrete-time systems
if ~exist('stick.mat'), 
    dstick
elseif exist('stick.mat') == 2,
    load stick
end

Ts = 0.01;  % in dstick already
disp('*******>'); pause; disp(' ');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Part B : design 1st-order unstable pendulum 
%          controller
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% locate the unstable pole at z = 1.0619
disp('The poles of the system are')
pole(Gz)
disp('*******>'); pause

% set up Kp(z)
disp('Controller Kp(z)')
Kp_zero = 0.96
Kp_pole = 1.01
Kpz = tf([1 -Kp_zero],[1 -Kp_pole],Ts)
disp('*******>'); pause

disp('Root-locus plot')
disp('Select Kpz to achieve a damping ratio of 0.8.')
figure, ucircle, hold on
rlocus(-Gz(1,1)*Kpz)
dzline(0.8), hold off
axis([0.8 1.1 -0.15 0.15])
[Kp_gain,ppp] = rlocfind(-Gz(1,1)*Kpz)
disp('The designed pendulum controller is')
% Kp_gain = 22.8; % recommended value
Kpz = -Kp_gain*Kpz;
disp('*******>'); pause

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Part C : design cart position controller
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Close the pendulum controller 
disp('System with pendulum loop closed')
Gz_pen_CL = feedback(Gz,Kpz,1,1)
disp('*******>'); pause

disp('Poles of system should include one at z = 1')
pole(Gz_pen_CL)
disp('Design the cart controller to move this z = 1 pole inside the unit circle')
disp('*******>'); pause

% use a proportional controller
disp('Root-locus plot')
disp('Select Kcz to achieve critical damping.')
figure, ucircle, hold on
rlocus(Gz_pen_CL(2,1)), hold off
axis([0.9 1.1 -0.05 0.05])
[Kcz,ppc] = rlocfind(Gz_pen_CL(2,1))
disp('*******>'); pause

disp('The closed-loop system is')
% Kcz = 0.6; % recommended value
Gz_CL = feedback(Gz_pen_CL*Kcz,1,1,2)
disp('*******>'); pause

disp('Step response to 0.1 m step command of cart position')
[y,dtime] = step(Gz_CL);

figure
plot(dtime,0.1*y*[180/pi 0; 0 1]); grid  
xlabel('Time (sec)')
ylabel('Pendulum angle (deg), cart position (m)')
title('Step response of closed-loop system: pendulum (blue), cart (green)')

% Done
disp('End of Comprehensive Problem CP8.2')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
